# -*- coding: utf-8 -*-    

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

import uuid

class EventListener:
    """ Interface of objects listening for events """
    def receive(self,  id,  event):
        """ This function will be called when this EventListener receives an event"""
        
    def knownEventTypes(self):
        """ This function returns the list of event types supported by this listener"""
        
    def registerToAllKnownEvents(self,  dispatcher):
        """Register this listener to all events it knows in an EventDispatcher"""
        dispatcher.registerForEvents(self.knownEventTypes())


class EventSender:
    """ Interface of objects sending events """
    def __init__(self):
        self.id = 0
        
    def registerToDispatcher(self,  dispatcher):
        """Registers this sender with an EventDispatcher"""
        self.id = dispatcher.getSenderId(self)
        
    def generatedEventTypes(self):
        """ Lists the event types that this sender can send"""



class EventDispatcher:
    def __init__(self):
        self.handlers = {}

    def registerForEvent(self,  listener,  eventType):
        if (self.handlers.has_key(eventType)):
            self.handlers[eventType].append(listener)
        else:
            self.handlers[eventType] = [listener, ]
            
    def registerForEvents(self,  listener,  eventTypes):
        for eventType in eventTypes:
            self.registerForEvent(listener,  eventType)
            
    def  sendEvent(self,  id,  event):
        if self.senders[id] is None:
            return
        for listener in self.handlers[event.type]:
            listener.receive(id,  event)
            
    def registerForAllEvents(self,  listener):
        for eventType in self.handlers.keys():
            self.registerForEvent(listener,  eventType)
            
    def unregister(self,  handler):
        for eventType in self.handlers.keys():
            self.handlers[eventType].remove(handler)
            
    def getSenderId(self,  sender):
        id = uuid.uuid4()
        self.senders[id] = sender
        
    def allEventTypes(self):
        return self.handlers.keys()
    
    
    
class Log (EventListener):
    def __init__(self,  conf):
        self.config = conf
        
    def receive(self):
        pass
        
