# -*- coding: utf-8 -*-    

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

#TODO : create an Exception object that will be raised to signal problems to the
# caller class (JSONserver), providing meaningful error messages for the user
#
# All DB transactions should be stored for "replay" purpose but this MUST NOT
#         provide a way to link voter and vote, even by just the timing info...
#
# Should sanitize user input (username, password, valid email address, date strings)
# 
# Add support for user rights per-scrutin : admin, voter, results reader, and a special Everyone user

import os,  hashlib,  datetime,  MySQLdb, json,  pickle

from pyvotons.core.ComputeResult import ScrutinComputeResult


#Custom JSON encoder
def PyVotonsJSONEncoder(o):
	if isinstance(o, datetime.datetime):
		return o.strftime("%Y-%m-%d %H:%M:%S")
	elif isinstance(o, datetime.date):
		return o.strftime("%Y-%m-%d")
	elif isinstance(o, datetime.time):
		return o.strftime("%H:%M:%S")
	else:
		return json.JSONEncoder.default(o)


class DataService:
	def __init__(self,  use_mysql = False,  db_url=None):
	        print "DataService init..."	
		self.use_mysql = use_mysql
		self.url = db_url
		self.lastScrutinsStateUpdate = datetime.datetime(day=1,month=1, year=2010)
		#Import
		if use_mysql:
			import MySQLdb as db_api
		else:
			import sqlite3 as db_api
		self.db_api = db_api
		self.connectDB()

                print "...init ok"
#		try:
#			func(cursor)
#			self.connection.commit()
#			cursor.close()

	def connectDB(self):
	  # Connect to the DB
	  if self.use_mysql:
		  print "      using mysql"
		  self.connection =  self.db_api.connect(host='localhost', passwd="pyvotons", db="pyvotons", user="pyvotons")
		  self.connection2 =  self.db_api.connect(host='localhost', passwd="pyvotons", db="pyvotons", user="pyvotons")
	  else:
		  print "      using sqlite"
		  self.connection =  self.db_api.connect(db_url)
		  self.connection2 =  self.db_api.connect(db_url)
	  # Get cursor
	  self.cursor = self.connection.cursor()
	  self.dictcursor = self.connection2.cursor(MySQLdb.cursors.DictCursor)
	  
	# Run a SQL query, restart the SQL connection if it fails and retry
	def cursorExecute(self, *args):
	  try:
	    self.cursor.execute(*args)
	  except (AttributeError, MySQLdb.OperationalError):
	    self.uninit()
	    self.connectDB()
	    self.cursor.execute(*args)
	  return self.cursor
	  
	def cursorExecutemany(self, *args):
	  try:
	    self.cursor.executemany(*args)
	  except (AttributeError, MySQLdb.OperationalError):
	    self.uninit()
	    self.connectDB()
	    self.cursor.executemany(*args)
	  return self.cursor
	
	
	def uninit(self):
		self.cursor.close()
		self.dictcursor.close()
		self.connection.close()
		self.connection2.close()



	def sqlDate(self,  date):
		"""Convert a datetime python object to a SQL datetime string"""
		return str(date).split('.')[0]

	def sqlNow(self):
		""" Returns a SQL formatted datetime for 'now'"""
		return str(datetime.datetime.now()).split('.')[0]
		
	def loginUser(self,  login, password):
		try:
			# Get the salt from the db
			self.cursorExecute("select salt from users where login=%s",  (login, ))
			salts = self.cursor.fetchone()
			if salts == None:
				print "Cannot get salt for login "+login
				return False
			# Compute the hash
			hash = hashlib.sha512( login + salts[0] + password).hexdigest()
			# Check if the hash is ok
			self.cursorExecute("select userid,login from users where login=%s and shapass=%s",  (login, hash))
			row = self.cursor.fetchone()
			if row is None:
				print "Incorrect password !"
				return False
			else:
				return row[0]
		except MySQLdb.Error, e:
			print "LoginUser Exception !"+str(e)
			return False
		
	def addUser(self,  dict):
		salt = ''.join(map(lambda x:'./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'[ord(x)%64], os.urandom(50)))
		hash = hashlib.sha512( dict['login']+ salt + dict['pass']).hexdigest()
		#Insert the new user
		try:
			# Check whether the username already exists
			self.cursorExecute("select login from users where login=%s", (dict['login'], ))
			if self.cursor.fetchone() is not None:
				print "Login already exists ! Cannot create user"
				return False
			self.cursorExecute("insert into users(login, firstname, lastname,  email, salt, shapass, privateemail, creationtime) values(%s, %s,%s,%s,%s,%s,%s,%s)",\
													(dict['login'], dict['firstname'],  dict['lastname'], dict['email'],  salt,  hash, dict['privateemail'],  self.sqlNow()))
			print "New user created !"
		except  MySQLdb.Error,  e:#MySQLdb.Error,
			print "AddUser failed : "+str(e)
			return False
		return True
		
	def deleteUser(self,  login):
		# Pour l'administrateur
		# TODO 
		# Si le user a deja fait quelque chose, il faut juste le desactiver, pas le détruire
		try:
			self.cursorExecute("delete from users where login=%s",  (login, ))
		except:
			print "Error deleting user"
			return False
		return True
		
		
	def listUsers(self):
		self.cursorExecute("select login, firstname, lastname, email, privateemail from users order by login asc")
		answ=[]
		while (1):
			row=self.cursor.fetchone()
			if row is None:
				break
			if row[4]:
				answ.append({'login':row[0], 'firstname':row[1], 'lastname':row[2], 'email':row[3] })
			else:
				answ.append({'login':row[0], 'firstname':row[1], 'lastname':row[2], 'email':'' })
		return answ

	def updateUser(self,  login,  newlogin=None, password=None,  firstname=None,  lastname=None,  email=None,  privateemail=None):
		self.cursorExecute("select login, salt, shapass, firstname, lastname, email, privateemail from users where login=%s",  (login, ))
		row = self.cursor.fetchone()
		if row is None:
			print "Cannot update unknown user "+login
			return False
		(oldlogin, salt, oldshapass, oldfirstname, oldlastname, oldemail, oldprivateemail) = row
		if not newlogin:
			newlogin=login
		if not password:
			shapass = oldshapass
		else:
			shapass = hashlib.sha512( newlogin + salt + password).hexdigest()
		
		if firstname is None:
			firstname=oldfirstname
		if lastname is None:
			lastname=oldlastname
		if email is None:
			email=oldemail
		if privateemail is None:
			privateemail=oldprivateemail
		self.cursorExecute("update users set login=%s, shapass=%s, firstname=%s, lastname=%s, email=%s where login=%s",  (newlogin,  shapass, firstname,  lastname,  email,  privateemail))
		if self.cursor.rowcount != 1:
			print "User update error : "+str(self.cursor.rowcount)+" rows updated"
			return False
		return True

	def addGroup(self,  name,  description,  public,  parentGroup=None):
		pass
	
	def updateGroup(self,  name, newname=None,   description=None,  public=None,  parentGroup=None):
		pass
		
	# Attention à ne pas supprimer un groupe qui est impliqué dans un scrutin, juste le désactiver
	def removeGroup(self,  group):
		pass
		
	def addMember(self,  user,  group):
		pass
		
	def removeMember(self, user,  group):
		pass
	
	def listMembers(self,  group):
		pass
		
	def setMemberStatus(self,  user,  group,  status):
		pass
		
	def getMemberStatus(self,  user,  group):
		pass
		
	def matchString(self,  datemin,  datemax,  titlewords,  descwords):
		pass

	def addLogEntry(self,  text,  level=0): # Log who did what and when for debug and forensics 
		pass
	
	# Create Scrutin and questions
	def addScrutin(self,  userid,  scrutin ):
		# Add a scrutin, add the questions to the db, add the voters to the voteregister
		self.connection.commit() # To allow a clean rollback if necessary
		scrutin['status'] = "valid" # SHOULD BE CHECKED
		try:
		  scrutin['openDate'] = datetime.strptime(scrutin['openDate'], '%Y/%m/%d').date()
		  scrutin['closeDate'] = datetime.strptime(scrutin['closeDate'], '%Y/%m/%d').date()
		except ValueError:
		  scrutin['openDate'] = None
		  
		if scrutin['openDate'] is None or scrutin['closeDate'] is None:
		  print "Date du nouveau scrutin invalide !"
		  return None
		  
		# FIXME CHECK OPENDATE < CLOSEDATE and the strings are valid dates (in the future, in less than a year, no invalid char...)
		try:
			self.cursorExecute("insert into scrutin (title, description, comment, creatorId,\
													registeredOnly, status, openDate, closeDate)\
																	values (%s, %s, %s, %s, %s, %s, %s, %s)", \
			(scrutin['title'],  scrutin['description'],  scrutin['commentsAllowed'],  userid,  scrutin['registeredOnly'],\
			scrutin['status'],  self.sqlDate(scrutin['openDate']),  self.sqlDate(scrutin['closeDate'])))
			scrutinid = self.cursor.lastrowid
			self.cursorExecutemany("insert into question (scrutinid, title, description, algo, type, idx, answers) values (%s, %s, %s, %s, %s, %s, %s)", \
									[(scrutinid,  q['title'],  q['description'],  q['algo'],  q['questionType'],  idx,  pickle.dumps(q['answers'])) for (q,  idx) in zip(scrutin['questions'],  range(len(scrutin['questions'])))])
			self.cursorExecutemany("insert into voteregister (scrutinid, userid) values (%s, %s)",  [(scrutinid,  userid) for userid in scrutin['voters']])
		except MySQLdb.Error, e:
			self.connection.rollback()
			print "Cancelling new Scrutin : DB Error " +str(e)
			return None
		return scrutinid
	
	# Changing opening/closing dates, cancelling/closing a scrutin ?
	def updateScrutin(self):
		# Should check that user has the rights to remove/update the scrutin
		pass # Destroy and recreate (to use all the checks provided by the create function ?)
		#Create a new scrutin, if ok, remove the old one and update the scrutinid to the old value.
		# Other way : modify scrutin creation function to handle updates
		# SQL : "update scrutin set openDate='2011-10-12',closeDate='2013-12-12' where scrutinid=7;"
	
	# Check if a scrutin is now open or closed
	def updateScrutinsState(self):
		# Select scrutins which are valid (waiting to open) or open (waiting to close) and check the dates
		# Do not recompute more often than every minute
		n = datetime.datetime.now()
		if n-self.lastScrutinsStateUpdate < datetime.timedelta(minutes=1):
			return
		self.lastScrutinsStateUpdate = n
		
		self.cursorExecute("select scrutinid, status, openDate, closeDate from scrutin where status='valid' or status='open'")
		rows = self.cursor.fetchall()
		toUpdate = []
		for r in rows:
			if r[1] == 'valid':
				if n>=r[2] and n <= r[3]: # The current time is after opening date and before closing
					toUpdate.append(('open',  r[0]))
				elif n > r[3]:
					toUpdate.append(('closed',  r[0]))
			elif r[1] == 'open' and  n > r[3]:
				toUpdate.append(('closed',  r[0]))
		
		if len(toUpdate) > 0:
			self.cursorExecutemany("update scrutin set status=%s where scrutinid=%s",  toUpdate)
			
	# Security ; matching should contain keys/value pairs and only known keys will be used to contruct the request
	# TODO Check if the user has rights to see each scrutin 
	def listScrutins(self,  userid,  filter=None):
		self.updateScrutinsState()
		filter = {'toVote':'where status=\'open\'',  'toSee':'',  'toResult':'where status=\'closed\' or status=\'open\''}[filter]
		self.cursorExecute("select scrutinid, title, status, openDate, closeDate from scrutin "+filter+" ORDER BY openDate ASC")
		rows = self.cursor.fetchall()
		return [dict(zip(['scrutinid','title', 'status', 'openDate', 'closeDate'],  row)) for row in rows]
	
	def getScrutin(self,  scrutinid,  userid):
		print "-->getScrutin"
		try:
			self.cursorExecute("select title, scrutinid, status, description, comment, creatorId,\
														registeredOnly, openDate, closeDate from scrutin where scrutinid=%s",  (scrutinid, ))
		except MySQLdb.Error, e:
			print "MYSQL error : "+str(e)
		except:
			print 'Error unknown'
		print "fetchall"
		res = self.cursor.fetchall()
		if len(res) != 1:
			print "Get Scrutin : there should be only one result and there is "+str(len(res))
			return False
		print "One scrutin was found : "+str(res[0][0])
		res = dict(zip(['title', 'scrutinid', 'status', 'description', 'commentsAllowed', 'creatorId', 'registeredOnly', 'openDate', 'closeDate'],  res[0]))
		if res['registeredOnly'] == True:
			pass# CHECK if userid is in the register
			
		# Get all questions #ORDER BY index ?
		print "Question ?"
		self.cursorExecute("select title, description, type, idx, answers from question where scrutinid=%s order by idx",  (scrutinid, ))
		ques = self.cursor.fetchall()
		print "Question ok"
		question = [dict(zip(['title', 'description', 'questionType', 'idx','answers'],  r)) for r in ques]
		for q in question:
			q['answers'] = pickle.loads(q['answers'])
		res['questions'] = question
		return res
		
	def getScrutinResult(self,  scrutinid,  userid):# Pour chaque question, 'results', 'explanation', 'winners'
		# If already computed and closed
		
		# If not computed or still ongoing and results too old, compute the result
		# CHECK access
		# Get all votes - according to the question order
		print "***getScrutinResult***"
		try:
			self.cursorExecute("select answers from votes where scrutinid=%s",  (scrutinid, ))
			answers = self.cursor.fetchall()
			print "ANSWERS "+repr(answers)
			answers = [pickle.loads(a[0]) for a in answers]
			# Get all questions - check the order !!
			self.cursorExecute("select title, description, type, algo, idx, answers from question where scrutinid=%s order by idx",  (scrutinid, ))
			questions = self.cursor.fetchall()
			questions = [dict(zip(['title', 'description', 'questionType', 'algo', 'idx', 'answers'],  q)) for q in questions]
			for q in questions:
				q['answers'] = pickle.loads(q['answers'])
			# Compute the results (each question returns a 'result', 'explanation' and 'winners'
			questionsWithResults = ScrutinComputeResult(questions,  answers)
			# Store the results
			# return
			return questionsWithResults
		except MySQLdb.Error, e:
			print "MYSQL error : "+str(e)
		
	def checkVote(self,  userid,  req):
		try:
			self.cursor.execute("select voteid from voteregister where scrutinid=%s and userid=%s",  (userid,  req['scrutin']['scrutinid']))
			if len(self.cursor.fetchall()) != 0:
				return "Already voted !"
		except MySQLdb.Error,  e:
			return str(e)
		except:
			return "Invalid request or user"
		return None

		#Difficile... Selon le type de scrutin, chaque type de question
		return ret
		
	# Add the user to the register and store the vote (2 steps, should be on different servers !)
	def registerVote(self,  userid,  req):
		try:
			self.cursor.execute("select registeredOnly from scrutin where scrutinid=%s",  (req['scrutin']['scrutinid'], ))
			res = self.cursor.fetchall()
			if len(res) != 1:
				print "Could not find registeredOnly in registerVote"
				return False
			if res[0] == True:
				self.cursor.execute("update voteregister set voteid=1 where scrutinid=%s and userid=%s",  (userid,  req['scrutin']['scrutinid']))
			else:
				self.cursor.execute("insert into voteregister (voteid, scrutinid, userid) values (1, %s, %s)",  (userid,  req['scrutin']['scrutinid']))
				
			self.cursor.execute("insert into votes(scrutinid,answers) values (%s,%s)",  (req['scrutin']['scrutinid'],  pickle.dumps(req['vote'])))
			return True
			
		except MySQLdb.Error,  e:
			print "RegisterVote MySQL error : "+str(e)
			return False
		
	# If the vote is not closed and not anonymous
	def updateVote(self):
		pass
#	
#		
#	def create_table(self):
#		self.cursor.execute(create_table_sql)
# 	self.connection.commit()
#	
#	def drop_table(self):
#		self.cursor.execute("drop table contact;")
#		self.connection.commit()
#	
#	def clear_table(self):
#		self.cursor.execute("delete from contact;")
#		self.connection.commit()
#		




