# -*- coding: utf-8 -*-    

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

#from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from SimpleSecureHTTPServer import SecureHTTPRequestHandler,  SecureHTTPServer
from SocketServer import ThreadingMixIn
from uuid import uuid4 as uuid
from gettext import gettext as _

import json, types, StringIO, traceback, re, os,  sys

# Don't forget export PYTHONPATH=/myhome/PyVotons/webPyVotons
from pyvotons.db.DataService import DataService,  PyVotonsJSONEncoder

# This class provides a basic server to listen for JSON requests. It then
# calculates the winner using the desired voting system and returns the results,
# again, encoded in JSON.

# TODO : the server is multithreaded, and there is only one datastore that contains a MySQL connection
# MySQLdb does NOT support multiple threads using the same connection.
# To avoid crashes, this should be corrected either using either a lock or a connection for each thread
# For saturation problems, there should be a limit on the number of threads

wwwDir = "/INSTALLDIR/pyvotons/ui/pyjamas/output/"
wordRe = re.compile('[\w_-]+')
fileRe = re.compile('[\w_-]+\.\w+')
filecache = {}
maxCache = 30
maxCacheFileSize = 3000000 # 3000 kb
cacheActive=False
clientData={}
datastore = DataService(True)

## WARNING : the cache mechanism can take a LOT of memory : do not serve big files !
class ElectionWebServiceHandler(SecureHTTPRequestHandler):

	def analyseRequest(self, req):
		print _("REQ received : ")+repr(req)
		# Should be in _init_
		if not hasattr(self, 'token'):
			self.token = -1
			
		if 'data' not in req or 'type' not in req:
			return {u'type':"error", u'errstr':u'Invalid request'}
			
		if 'token' in req['data']:
			self.token = req['data']['token']
		if not hasattr(self, 'location'):
			self.location = -1
		if self.token  not in clientData and req[u'type'] != 'UI' and req[u'type'] != 'authenticate'and req[u'type'] != 'createUser':
			return {u'type':"error", u'errstr':u'You are not identified !', u'event':req}
			
		# UI requests
		if req[u'type'] == u"UI":
			if u'goto' in req[u'data']:
				if self.token  not in clientData:
					if req[u'data'][u'goto'] == u"auth" or req[u'data'][u'goto'] == u"newuser":
						self.location = req[u'data'][u'goto'] 
						return {u'type':u"apply", u'event':req}
					else:
						return {u'type':"error", u'errstr':u'You are not identified !', u'event':req}
				else: # Valid token and goto -> just apply
					self.location = req[u'data'][u'goto']
					clientData[self.token]['location'] = self.location
					# For some cases, get data : QuickVote/QuickResult/updateScrutin -> get the scrutin data from selectedScrutinid
					if self.location in ['QuickVote',  'QuickResult',  'updateScrutin']:
						#try:
						if self.location =='QuickResult':
							scrutin = datastore.getScrutin(req[u'data']['selectedScrutinid'],  clientData[self.token]['userid'])
							scrutin['questions'] = datastore.getScrutinResult(req[u'data']['selectedScrutinid'],  clientData[self.token]['userid'])
						else:
							scrutin = datastore.getScrutin(req[u'data']['selectedScrutinid'],  clientData[self.token]['userid'])
						#except:
						#	print "Could not get scrutin data for id = "+str(req[u'data']['selectedScrutinid'])+", uid :"+str(clientData[self.token]['userid'])
						#	print "Error:", repr(sys.exc_info())
						#	return {u'type':"error", u'errstr':u'Selected Scrutin ID is invalid', u'event':req}
						req[u'data']['scrutin'] = scrutin
						return {u'type':u"apply", u'event':req}
					print "GOTO : "+str(self.location)
					return {u'type':u"apply", u'event':req}
			else:# UI, not goto
				pass
				
				
		elif req[u'type'] == u"authenticate":
			if req[u'data'][u'action'] == u'login':
				print 'Loging in...'
				# CHECK in database
				userid = datastore.loginUser(req[u'data']['login'],  req[u'data']['pass'])
				if not userid:
					return  {u'type':u"error", u'errstr':_(u'Login or password unknown'), u'event':req}
				self.token = str(uuid())
				clientData[self.token]={'location':None,  'userid':userid}
				return {u'type':u'ok', u'event':req, u'token':self.token}
			elif req[u'data'][u'action'] == u'logout':
				print "Loging out"
				if req[u'data'][u'token'] in clientData:
					del clientData[req[u'data'][u'token']]
				return {u'type':u'ok', u'event':req, u'token':None}
			else:
				print 'authenticate unknown'
				return {u'type':u"error", u'errstr':u'Authentication request type unknown', u'event':req}
				
				
		elif req[u'type'] == u"createUser":
			print "Creating user "+req[u'data']['login']
			added = datastore.addUser(req[u'data'])
			if added:
				return {u'type':u'ok', u'event':req, u'token':None}
			else:
				return  {u'type':u"error", u'errstr':u'New user was not created', u'event':req}

		elif req[u'type'] == u"updateUser":
			print "Updating user "+req[u'data']['login']
			updated = datastore.updateUser(req[u'data'])
			if updated:
				return {u'type':u'ok', u'event':req, u'token':self.token}
			else:
				return  {u'type':u"error", u'errstr':u'User was not updated', u'event':req}

		elif req[u'type'] == u"listUser":
			return   {u'type':u"error", u'errstr':u'LIST users is not implemented', u'event':req}
			
		elif req[u'type'] == u"listScrutins":
			scr = datastore.listScrutins(clientData[self.token]['userid'],  req[u'data']['goal'])
			return {u'type':u'ok', u'event':req, u'token':self.token,  'scrutins':scr}
			
		elif req[u'type'] == u"createScrutin":
			print "ADD SCRUTIN"
			created = datastore.addScrutin(clientData[self.token]['userid'],  req[u'data'])
			if created is not None:
				return {u'type':u'ok', u'event':req, u'token':self.token,  'scrutinid':created}
			else:
				return  {u'type':u"error", u'errstr':u'Scrutin was not created', u'event':req}
		
		elif req[u'type'] == u"checkAndRegisterVote":
			# Best way should be : check the userid and the voters' list, then send back a UUID (also stored locally as "ok for one vote")
			# The client can then send an anonymous vote to the server (no user token, no userid, just a	 vote and the uuid)
			print "Check and Register Vote"
			check = datastore.checkVote(clientData[self.token]['userid'],  req[u'data'])
			if check is not None:
				print "Invalid vote "+str(check)
				return  {u'type':u"error", u'errstr':u'Vote is invalid :'+str(check), u'event':req}
			if datastore.registerVote(clientData[self.token]['userid'],  req[u'data']):
				return {u'type':u'ok', u'event':req,  u'token':self.token}
			else:
				return  {u'type':u"error", u'errstr':u'Could not register vote !', u'event':req}
			
		else:# Neither authenticate, createUser nor UI
			print "Not implemented"
			return {u'type':u"error", u'errstr':u'Request type unknown', u'event':req}

	def do_GET(self):
	  try:
            print "GET received"
	    if len(self.path) > 50:
	      raise IOError
	    path = str(self.path).split('/')
	    print "PATH = "+str(path[-1])
	    filename = path[-1].split('?')[0]
	    print 'FILENAME = '+repr(path[-1].split('?')[0])
	    if not re.match(fileRe, filename):
	      print "Invalid filename"
	      raise IOError

	    for p in path[1:-1]:
	      if not re.match(wordRe, p):
                print "Directory name invalid:"+str(p)
                raise IOError
              
	    filename = self.path.split('?')[0]
	    if filename not in filecache or  cacheActive is False:
	      f = open(wwwDir+filename) 
	      if len(filecache) > maxCache: # Do not fill the whole memory with files...
	        (oldname, oldfile)=filecache.popitem()
                del oldfile
                print 'Destroying cache for :'+ str(oldname)
              print 'caching '+str(filename)
	      filecache[filename]=f.read()
	      f.close()
	    else:
	      print 'Sending cached file '+str(filename)

	    self.send_response(200)
	    if filename.endswith(".html"):
                self.send_header('Content-type', 'text/html')
	    elif filename.endswith(".css"):
            	self.send_header('Content-type', 'text/css')
	    elif filename.endswith(".js"):
            	self.send_header('Content-type', 'text/javascript') #"text/javascript" or "application/javascript"
	    elif filename.endswith(".png"):
            	self.send_header('Content-type', 'image/png')
	    elif filename.endswith(".jpg") or filename.endswith(".jpeg"):
            	self.send_header('Content-type', 'image/jpeg')
	    elif filename.endswith(".gif"):
            	self.send_header('Content-type', 'image/gif')
	    else:
            	print "Unknown filetype : $filename"
            	self.send_header('Content-type',    'text/plain')
	      
	    self.end_headers()
	    # self.wfile.write(f.read())
	    self.wfile.write(filecache[filename])

            # Do not cache large files !
	    if len(filecache[filename]) > maxCacheFileSize:
	      print filename+' is too big for the cache -> deleted'
	      del filecache[filename]

	    #f.close()
	    return

          except IOError:
            self.send_error(404,'File Not Found: %s' % self.path)
	
	def do_POST(self):
#		try:
			print "POST received"	
			# Parse the incoming data
			request_raw = self.rfile.read(int(self.headers["content-length"]))
            		print "RECEIVED : "+repr(request_raw)
#REQUEST RECEIVED : '{"jsonrpc":"2.0","id":"ID2","method":"request","params":[{"type":"UI","data":{"goto":"auth","token":null}}]}'
			request_raw = re.sub("\n", "", request_raw)
			request = json.loads(request_raw)
			response = {'result':{'type':"error", 'data':"hello"}}
			if request['method'] == 'request':
				# Analyse the request
				print "Request :"+ repr(request['params'][0])
				req = request['params'][0]
				result = self.analyseRequest(req)
				response['result'] = result
				print "Response is "+repr(result)
			else:
				print "Unknown message received !"

			print "Sending response..."
			response = json.dumps(response,  default=PyVotonsJSONEncoder)
			self.wfile.write(response)
			print "Response sent !"
			
#		except:
#			print "ERROR with the message REQUEST"
#			self.send_response(500)
#			self.wfile.write(json.dumps({'error':'unknown server error'}))

	
class ThreadedHTTPServer(ThreadingMixIn, SecureHTTPServer):
	pass

def main():
	try:
		server = ThreadedHTTPServer(('localhost', 8000), ElectionWebServiceHandler)
		print('Webservice running... https://localhost:8000/MainMenuPage.html')
		server.serve_forever()
	except KeyboardInterrupt:
		print('Webservice stopping...')
		server.socket.close()

if __name__ == '__main__':
	main()
