# -*- coding: utf-8 -*-    

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

# TODO pyjamas/addons/RichTextEditor.py might be of interest...

from pyjamas import Window
from pyjamas.ui.Button import Button
from pyjamas.ui.HTML import HTML
from pyjamas.ui.VerticalPanel import VerticalPanel
from pyjamas.ui.HorizontalPanel import HorizontalPanel
from pyjamas.ui.TextBox import TextBox
from pyjamas.ui.RichTextArea import RichTextArea
from pyjamas.ui.CheckBox import CheckBox
from pyjamas.ui.ListBox import ListBox


from pyvotons.core.event import Event

from pyvotons.ui.pyjamas.ChooseQuestionPanel import ChooseQuestionPanelEditor
from pyvotons.ui.pyjamas.SelectQuestionPanel import SelectQuestionPanelEditor
from pyvotons.ui.pyjamas.OrderQuestionPanel import OrderQuestionPanelEditor,  addWidgets,  getWidgetsData
from pyvotons.ui.pyjamas.RateQuestionPanel import RateQuestionPanelEditor

import pyjd

class CreateScrutinPage (VerticalPanel):
    def __init__(self,  mainpanel = None,  token = None):
        VerticalPanel.__init__(self)

        if token is None:
            self.token = -1
        else:
            self.token = token
            
        if mainpanel is None:
            Window.alert("CreateScrutin : mainpanel is not valid !")
            return
        self.mainpanel = mainpanel
        self.add(HTML("<h3><center>Create a new scrutin</center></h3>"))
        self.widg={'title':'TextBox', 'description':'RichTextArea','commentsAllowed':'CheckBox',  'registeredOnly':'CheckBox', 'openDate': 'DateTime',  'closeDate':'DateTime'}
        self.widgets = addWidgets(self,  self.widg)
        self.questionsPanel = VerticalPanel()
        self.questionsPanel.add(HTML("Questions :"))
        self.questions=[]
        addQuestionPanel = HorizontalPanel()
        addQuestionPanel.add(Button("Add a question",  self.addQuestion))
        self.questionTypeCombo = ListBox(VisibleItemCount=1)
        for i in ['Choose one',  'Select one or more',  'Order',  'Rate']:
			self.questionTypeCombo.addItem(i)
        addQuestionPanel.add(self.questionTypeCombo)
        addQuestionPanel.add(Button("Remove last question",  self.removeLastQuestion))
        self.add(self.questionsPanel)
        self.add(addQuestionPanel)
        self.add(Button("Create the scrutin",  self.clickCreate))

    def addQuestion(self):
		# Check self.questionTypeCombo.getValue()
		# Add the appropriate QuestionPanel to the questionsPanel and to self.questions
		q = {'Choose one':ChooseQuestionPanelEditor,  'Select one or more':SelectQuestionPanelEditor,\
       		'Order':OrderQuestionPanelEditor,  'Rate':RateQuestionPanelEditor}\
			   [self.questionTypeCombo.getSelectedItemText()[0]]()
		self.questions.append(q)
		self.questionsPanel.add(q)

    def removeLastQuestion(self):
		# Warn the user, then remove the question from the questionsPanel and from self.questions
		if len(self.questions) == 0:
			return
		if not Window.confirm("Are you sure you want to remove the last question ?"):
			return
		self.questionsPanel.remove(self.questions[-1])
		del self.questions[-1]
		
				
    def clickCreate(self):
		self.mainpanel.send(Event("createScrutin",  self.getData(self.widg)))
	
    def getData(self,  widg):
		print "GETDATA "+repr(widg)
		ret = getWidgetsData(self.widgets,  widg)
		ret['questions'] = [repr(q) for q in self.questions]	
		ret['voters'] = [0]
		return ret
			
				
    def tableFromScrutins(self,  scrutins):
		"""Makes a Grid from a list of scrutin dictionaries"""
		rows = len(scrutins)
		if rows < 1:
			return HTML("No scrutin to display")
		columns = len(scrutins[0])
		keys = scrutins[0].keys()
		g=Grid(rows+1, columns)
		g.addClickListener(getattr(self, "onGridClick"))
		# Title row
		z=[g.setText(0, i, "<b>"+keys[i]+"</b>") for i in range(columns)]
		#Other rows
		for r in range(1, rows+1):
			z=[g.setText(r, i, "<b>"+scrutins[r-1][i]) for i in range(columns)]

    def onGridClick(self,  event=None):
		Window.alert("ON GRID Click : "+repr(event))


