# -*- coding: utf-8 -*-

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

from pyjamas import Window
from pyjamas.ui.RootPanel import RootPanel
from pyjamas.ui.VerticalPanel import VerticalPanel
from pyjamas.ui.HTML import HTML

from pyvotons.core.event import Event
from pyvotons.core.sanitize import  *
from pyvotons.ui.pyjamas.ChooseQuestionPanel import ChooseQuestionResultPanel
from pyvotons.ui.pyjamas.SelectQuestionPanel import SelectQuestionResultPanel
from pyvotons.ui.pyjamas.OrderQuestionPanel import OrderQuestionResultPanel
from pyvotons.ui.pyjamas.RateQuestionPanel import RateQuestionResultPanel


import pyjd



## This class displays a Panel allowing to display the results of a vote without loging in

class QuickResult  (VerticalPanel):
    def __init__(self,  mainpanel=None,  voteId = None,  scrutin=None):
        VerticalPanel.__init__(self,  StyleName="pyv-PageBody")
        self.mainpanel = mainpanel
        Window.alert("QuickResult : SCRUTIN = "+repr(scrutin))
        # Parametre de la page, on cherche le vodeId
        if voteId is None and scrutin is None:
            myDic = Window.getLocation().getSearchDict()
            if 'voteId' in myDic:
                voteId = myDic['voteId']
            else:
                self.add(HTML("<h1> Numéro de vote Invalide !</h1>"))
                return
        elif voteId is None and scrutin is not None:
            self.displayVoteResults(scrutin)
            return
        # Ask the server to get the vote results
        try:
            self.voteId = sanitizeAlphanum(voteId)
            self.getVoteResults(voteId)
        except:
            self.add(HTML("Impossible d'accéder aux résultats du vote "))
            return
        
        
    def getVoteResults(self,  id):
		if self.mainpanel:
			self.mainpanel.send(Event('getResult',  {'id':id}),  self.onResultsReceived,  self.onErrorReceived)
		else:
			self.add(HTML("Cannot contact server directly - NOT IMPLEMENTED"))

    def displayVoteResults(self,  s):
        panelBuilder = {'Choose': lambda  q: ChooseQuestionResultPanel(q), \
                        'Select': lambda q: SelectQuestionResultPanel(q), \
                        'Rate':lambda q: RateQuestionResultPanel(q), \
                        'Order':lambda q: OrderQuestionResultPanel(q)}
        self.add(HTML("<h1>Résultats du scrutin %s</h1>" % (s['title'])))
        self.add(HTML("<h2>%s</h2>" % (s['description'])))
        for ques in s['questions']:
            if ques['questionType'] in panelBuilder:
                self.add(panelBuilder[ques['questionType']](ques))
            else:
                self.add(HTML("Question de type invalide ! - ignorée"))

    def onResultsReceived(self,  res):
        # Créer les panneaux correspondant aux types des questions
        if res is None:
          self.add(HTML("<h1>Empty results received from server !</h1>"))
          return
        s = res['scrutin']
        self.displayVoteResults(s)
		
		
    def onErrorReceived(self,  err):
		self.add(HTML("Erreur du serveur"))


if __name__ == '__main__':
    pyjd.setup("./QuickResult.html")
    RootPanel().add(QuickResult())
    pyjd.run()
    

