# -*- coding: utf-8 -*-

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

from pyjamas.ui.VerticalPanel import VerticalPanel
from pyjamas.ui.HorizontalPanel import HorizontalPanel
from pyjamas.ui.DisclosurePanel import DisclosurePanel
from pyjamas.ui.TextBox import TextBox
from pyjamas.ui.TextArea import TextArea
from pyjamas.ui.RichTextArea import RichTextArea
from pyjamas.ui.CheckBox import CheckBox
from pyjamas.ui.HTML import HTML
from pyjamas.ui.ListBox import ListBox
from pyjamas.ui.Button import Button
from pyjamas.ui.Grid import Grid
from pyvotons.ui.pyjamas.OrderQuestionPanel import addWidgets,  getWidgetsData

# TODO : number input is HARD (no widget in pyjamas) -> Combobox
# Or use Widget.py NumericTextBox from https://code.google.com/p/pyjamas-utils/source/checkout
class RateQuestionPanel(VerticalPanel):
    def __init__(self,  question):
        VerticalPanel.__init__(self)
        # Sanity checks
        if 'questionType' not in question or question['questionType'] != "Rate":
            self.add(HTML("<h1>Erreur ! La question n'est pas du type Rate</h1>"))
            return
        self.add(HTML(question['title']))
        self.add(HTML(question['description']))
        self.rates = []
        for answer in question['answers']:
			hp = HorizontalPanel()
			hp.add(HTML(answer))
			r = ListBox(VisibleItemCount=1)
			for i in range(question['rateMin'], question['rateMax']):
				r.addItem(str(i))
			hp.add(r)
			self.rates.append(r)
			self.add(hp)
            
    def status(self):
        return [r.getValue() for r in self.rates]

class RateQuestionPanelEditor(VerticalPanel):
	def __init__(self,  question):
		VerticalPanel.__init__(self)
		self.add(HTML("Rate QUESTION EDITOR"))
		self.widg={'title':'TextBox',  'description':'RichTextArea',   'algo':'ComboBox', 'answersPanel':'VerticalPanel'}
		self.widgets = addWidgets(self,  self.widg)
		for i in ['Majority', 'MajoritySharedVote', 'SingleTransferableVote', 'MeanRate', 'MeanSharedVote',  'MedianRate', 'MedianSharedVote' ]:
			self.widgets['algo'].addItem(i)
		questionButtonsPanel = HorizontalPanel()
		questionButtonsPanel.add(Button("Add an answer",  self.addAnswer))
		questionButtonsPanel.add(Button("Remove selected answers",  self.removeSelectedAnswers))
		self.add(self.widgets['answersPanel'])
		self.add(questionButtonsPanel)
		self.answers = []
		
	def addAnswer(self):
		a = (CheckBox(),  TextBox('Answer text'),  HorizontalPanel())
		self.answers.append(a)
		hp = a[2]
		hp.add(a[0])
		hp.add(a[1])
		self.widgets['answersPanel'].add(hp)
		
	def removeSelectedAnswers(self):
		for a in self.answers:
			if a[0].isChecked():
				self.widgets['answersPanel'].remove(a[2]) 
		self.answers = [a for a in self.answers if not a[0].isChecked() ]
		
	def getAnswers(self):
		return [a[1].getText() for a in self.answers]
	
	def __repr__(self):
		return {'questionType':'Rate',  'title':self.widgets['title'].getText(),\
		        'algo':self.widgets['algo'].getSelectedItemText()[0], \
          		'description':self.widgets['title'].getText(), 'answers':self.getAnswers(), \
				'rateMin':0,  'rateMax':20}


class RateQuestionResultPanel(VerticalPanel):
    def __init__(self,  question,  displayMode = 'full'):
        VerticalPanel.__init__(self)
        # Sanity checks
        if 'questionType' not in question or question['questionType'] != "Rate":
            self.add(HTML("<h1>Erreur ! La question n'est pas du type Rate</h1>"))
            return
        if displayMode == 'full':
            self.fullResults(question)
        else:
            self.shortResults(question)
    
    def fullResults(self,  question):
        VerticalPanel.clear(self)
        self.add(HTML(question['title']))
        grid = Grid(2, len(question['answers']),
                    BorderWidth=2,
                    CellPadding=4,
                    CellSpacing=1)
        idx = 0
        for (answer, res) in question['results']:
            if (answer,  res) in question['winners']:
                grid.setHTML(0,  idx, "<font color='green'>"+answer+"</font>")
            else:
                grid.setHTML(0,  idx, answer)
                
            grid.setHTML(1, idx, str(res))
            idx += 1
        
        self.add(grid)
        disc = DisclosurePanel('Details')
        disc.setHeader(HTML("<b>Détails du calcul...</b>"))
        disc.setContent(HTML(question['explanation']))
        self.add(disc)
		
    def shortResults(self,  question):
        VerticalPanel.clear(self)
        self.add(HTML(question['title']))
        grid = Grid(2, len(question['winners']),
                    BorderWidth=2,
                    CellPadding=4,
                    CellSpacing=1)
        idx = 0
        for (answer, res) in question['winners']:
            grid.setHTML(0,  idx, "<font color='green'>"+answer+"</font>")
            grid.setHTML(1, idx, str(res))
            idx += 1
        self.add(grid)
    
