# -*- coding: utf-8 -*-

# Copyright 2009-2010 Manik Bhattacharjee (manik-listes at altern.org)
#                                                                
# This file is part of PyVotons!.                                
#                                                                
# PyVotons! is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or   
# (at your option) any later version.                                 
#                                                                     
# PyVotons! is distributed in the hope that it will be useful,        
# but WITHOUT ANY WARRANTY; without even the implied warranty of      
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
# GNU Affero General Public License for more details.                        
#                                                                     
# You should have received a copy of the GNU Affero General Public License   
# along with PyVotons!.  If not, see <http://www.gnu.org/licenses/>. 

from pyjamas import Window
from pyjamas.ui.VerticalPanel import VerticalPanel
from pyjamas.ui.DisclosurePanel import DisclosurePanel
from pyjamas.ui.Grid import Grid
from pyjamas.ui.Button import Button
from pyjamas.ui.HorizontalPanel import HorizontalPanel
from pyjamas.ui.TextBox import TextBox
from pyjamas.ui.TextArea import TextArea
from pyjamas.ui.RichTextArea import RichTextArea
from pyjamas.ui.CheckBox import CheckBox
from pyjamas.ui.ListBox import ListBox
from pyjamas.ui.HTML import HTML
from pyjamas.ui.Button import Button
from pyvotons.ui.pyjamas.OrderQuestionPanel import addWidgets,  getWidgetsData

# TODO Vote blanc possible ou non ?


class SelectQuestionPanel(VerticalPanel):
    def __init__(self,  question):
        VerticalPanel.__init__(self,  StyleName="pyv-QuestionPanel")
        # Sanity checks
        if 'questionType' not in question or question['questionType'] != "Select":
            self.add(HTML("<h1>Erreur ! La question n'est pas du type Select</h1>"))
            return
        # Fill the layout and keep track of the answers' checkboxes
        self.box=[]
        self.add(HTML(question['title']))
        self.add(HTML(question['description']))
        for answer in question['answers']:
            b = CheckBox(answer)
            self.box.append(b)
            b.addClickListener(getattr(self, "onClick"))
        z=[self.add(b) for b in self.box]
        
    def onClick(self, sender=None):
            Window.alert("Checkbox was clicked !" + str(sender.isChecked()))
    def status(self):
        return [b.isChecked() for b in self.box]
        

class SelectQuestionPanelEditor(VerticalPanel):
	def __init__(self,  question):
		VerticalPanel.__init__(self,  StyleName="pyv-QuestionPanel")
		self.add(HTML("Select one or more QUESTION EDITOR"))
		self.widg={'title':'TextBox',  'description':'RichTextArea',   'algo':'ComboBox', 'answersPanel':'VerticalPanel'}
		self.widgets = addWidgets(self,  self.widg)
		for i in ['Majority','MajoritySharedVote' ]: # Shared vote : one voice dispatched between selected answers vs Majority : one voice is added to each selected answer
			self.widgets['algo'].addItem(i)
		questionButtonsPanel = HorizontalPanel()
		questionButtonsPanel.add(Button("Add an answer",  self.addAnswer))
		questionButtonsPanel.add(Button("Remove selected answers",  self.removeSelectedAnswers))
		self.add(self.widgets['answersPanel'])
		self.add(questionButtonsPanel)
		self.answers = []
		
	def addAnswer(self):
		a = (CheckBox(),  TextBox('Answer text'),  HorizontalPanel())
		self.answers.append(a)
		hp = a[2]
		hp.add(a[0])
		hp.add(a[1])
		self.widgets['answersPanel'].add(hp)
		
	def removeSelectedAnswers(self):
		for a in self.answers:
			if a[0].isChecked():
				self.widgets['answersPanel'].remove(a[2]) 
		self.answers = [a for a in self.answers if not a[0].isChecked() ]
		
	def getAnswers(self):
		return [a[1].getText() for a in self.answers]
	
	def __repr__(self):
		return {'questionType':'Select',  'title':self.widgets['title'].getText(),  'algo':self.widgets['algo'].getSelectedItemText()[0],  'description':self.widgets['title'].getText(), 'answers':self.getAnswers()}


class SelectQuestionResultPanel(VerticalPanel):

    def __init__(self,  question,  displayMode = 'full'):
        VerticalPanel.__init__(self,  StyleName="pyv-QuestionPanel")
        # Sanity checks
        if 'questionType' not in question or question['questionType'] != "Select":
            self.add(HTML("<h1>Erreur ! La question n'est pas du type Select</h1>"))
            return
        # Fill the layout and keep track of the answers' checkboxes
        if displayMode == 'full':
            self.fullResults(question)
        else:
            self.shortResults(question)


    def fullResults(self,  question):
        VerticalPanel.clear(self)
        self.add(HTML(question['title']))
        grid = Grid(2, len(question['answers']),
                    BorderWidth=2,
                    CellPadding=4,
                    CellSpacing=1)
        idx = 0
        for (answer,  res) in question['results']:
            if [answer,  res] in question['winners']:
                grid.setHTML(0,  idx, "<font color='green'><b>"+answer+"</b></font>")
            else:
                grid.setHTML(0,  idx, answer)
                
            grid.setHTML(1, idx, "%d" % res)
            idx += 1
        
        self.add(grid)
        disc = DisclosurePanel('Details')
        disc.setHeader(HTML("<b>Détails du calcul...</b>"))
        disc.setContent(HTML(question['explanation']))
        self.add(disc)

    def shortResults(self,  question):
        VerticalPanel.clear(self)
        self.add(HTML(question['title']))
        grid = Grid(2, len(question['winners']),
                    BorderWidth=2,
                    CellPadding=4,
                    CellSpacing=1)
        idx = 0
        for (answer,  res) in question['winners']:
            grid.setHTML(0,  idx, "<font color='green'>"+answer+"</font>")
            grid.setHTML(1, idx, str(res))
            idx += 1
        self.add(grid)


